from chimp_osc import ChimpOSCInterface
from chimp_osc.console.button import Button

class Fader:
    def __init__(self, itf: ChimpOSCInterface, nr : int):
        self._itf = itf
        self._nr = nr
        self._name = None
        self._value = 0
        self.go = Button(lambda value: self._itf.fader_go(self._nr, value))
        self.pause = Button(lambda value: self._itf.fader_pause(self._nr, value))
        self.flash = Button(lambda value: self._itf.fader_flash(self._nr, value))
        self._itf.add_handler(f"/chimp/fader/{self._nr}/value",self._handle_osc_value)
        self._itf.add_handler(f"/chimp/fader/{self._nr}/name",self._handle_osc_name)

    def _handle_osc_value(self, address, *args):
        assert isinstance(args[0],float)
        self._value = args[0]

    def _handle_osc_name(self, address, *args):
        assert isinstance(args[0],str)
        self._name = args[0]

    @property
    def value(self):
        return self._value/10
    
    @value.setter
    def value(self, value: float):
        self._value =max(min(value,100),0)*10
        self._itf.fader_value(self._nr,self._value)

    @property
    def name(self):
        return self._name
    
    @property
    def display(self):
        return self.name if self.name else self._nr